
`timescale 1ns / 1ps

module Clock_Man(EXTCLK,DCM_RST,POW_ON_RST,
                 REFCLK,USRCLK,DCM_L_RIO,DCM_L_SYS,SYSCLK,
                 VME_OSC,VMECE,VME_SYSCLK,SER_CLK,DCM_L_SER,
                 DV1024CLK,CK_3P9MS);

     input EXTCLK;
     input DCM_RST;
     input POW_ON_RST;
    output REFCLK;
    output USRCLK;
    output DCM_L_RIO;
    output DCM_L_SYS;
    output SYSCLK;
     input VME_OSC;
     input VMECE;
    output VME_SYSCLK;
    output SER_CLK;
    output DCM_L_SER;
//    output OFF_CLK;
//     input SMODE;
    output DV1024CLK;
    output CK_3P9MS;

//    wire clkdv2;
    wire clk1x_u1;
//    wire clkfb_u1;
    wire clk1x_u2;
    wire clkfb_u2;
//    wire clk2x_w;
    wire clk4x_w;
//    wire clk4x180_w;
    wire vmeclkfb_w;
    wire vmeclk1x_w;
    wire vme_in;
    wire serclk_w;
    wire serclk1x_w;
    wire serfb_w;
    wire GSR;
    wire GND;
wire DV32C;
wire DV32CLK;
wire DV1024C;
wire DV1024CLK;
wire IMC;
wire IMCLK;
wire MSC;
wire CK_3P9MS;

//reg DV2,DV4,DV8,DV16,DV32,DV64,DV128,DV256,DV512,DV1024;
//reg [6:0] DV_CNT_DWN;

assign GND = 1'b0;
// synthesis translate_off
assign GSR = glbl.GSR;         // Add in for functional simulations; remove for post synthesis simulations
// synthesis translate_on


    BUFG    INCLKBUF (.I(EXTCLK),.O(REFCLK));
    BUFG    RIOBUF (.I(clk1x_u1),.O(USRCLK));

`ifdef Simulation
    // Attributes for functional simulation//
    // synthesis translate_off
       defparam U1_DCM.CLK_FEEDBACK          = "1X";
       defparam U1_DCM.DLL_FREQUENCY_MODE    = "LOW";
       defparam U1_DCM.DFS_FREQUENCY_MODE    = "LOW";
       defparam U1_DCM.DUTY_CYCLE_CORRECTION = "TRUE";
       defparam U1_DCM.STARTUP_WAIT          = "TRUE";
    // synthesis translate_on
`endif

// attribute declarations for synthesis

//synthesis attribute          CLK_FEEDBACK of U1_DCM is "1X" 
//synthesis attribute    DLL_FREQUENCY_MODE of U1_DCM is "LOW" 
//synthesis attribute    DFS_FREQUENCY_MODE of U1_DCM is "LOW" 
//synthesis attribute DUTY_CYCLE_CORRECTION of U1_DCM is "TRUE" 
//synthesis attribute          STARTUP_WAIT of U1_DCM is "TRUE" 

    // Instantiate the DCM primitive//
         DCM U1_DCM ( 
               .CLKIN(REFCLK), 
               .CLKFB(USRCLK), 
               .DSSEN(GND), 
               .PSCLK(GND), 
               .PSEN(GND), 
               .PSINCDEC(GND), 
               .RST(DCM_RST), 
               .CLK0(clk1x_u1),
               .LOCKED(DCM_L_RIO)
              );

///////////////////////////////////
//                               //
// High frequency clock          //
//                               //
///////////////////////////////////

    BUFG    FBU2BUF (.I(clk1x_u2),.O(clkfb_u2));

    BUFG    SYSBUF (.I(clk4x_w),.O(SYSCLK));

/*
    BUFG    SYS180BUF (.I(clkfx180_w),.O(SYSCLK180));
*/
`ifdef Simulation
    // Attributes for functional simulation//
    // synthesis translate_off
       defparam U2_DCM.CLKIN_PERIOD          = 16.0;
       defparam U2_DCM.CLK_FEEDBACK          = "1X";
//       defparam U2_DCM.DLL_FREQUENCY_MODE    = "LOW";
//       defparam U2_DCM.DFS_FREQUENCY_MODE    = "LOW";
       defparam U2_DCM.DLL_FREQUENCY_MODE    = "HIGH";
       defparam U2_DCM.DFS_FREQUENCY_MODE    = "HIGH";
       defparam U2_DCM.DUTY_CYCLE_CORRECTION = "TRUE";
       defparam U2_DCM.STARTUP_WAIT          = "FALSE";
       defparam U2_DCM.CLKFX_MULTIPLY        = 4;
       defparam U2_DCM.CLKFX_DIVIDE          = 1;
//       defparam U2_DCM.CLKFX_MULTIPLY        = 8;
//       defparam U2_DCM.CLKFX_DIVIDE          = 3;

// synthesis translate_on
`endif

// attribute declarations for synthesis

//synthesis attribute          CLKIN_PERIOD of U2_DCM is "16.0" 
//synthesis attribute          CLK_FEEDBACK of U2_DCM is "1X" 
//synthesis attribute    DLL_FREQUENCY_MODE of U2_DCM is "HIGH" 
//synthesis attribute    DFS_FREQUENCY_MODE of U2_DCM is "HIGH" 
//synthesis attribute DUTY_CYCLE_CORRECTION of U2_DCM is "TRUE" 
//synthesis attribute          STARTUP_WAIT of U2_DCM is "FALSE" 
//synthesis attribute        CLKFX_MULTIPLY of U2_DCM is "4" 
//synthesis attribute          CLKFX_DIVIDE of U2_DCM is "1" 

    // Instantiate the DCM primitive//
         DCM U2_DCM ( 
               .CLKIN(REFCLK), 
               .CLKFB(clkfb_u2), 
               .DSSEN(GND), 
               .PSCLK(GND), 
               .PSEN(GND), 
               .PSINCDEC(GND), 
               .RST(DCM_RST), 
               .CLK0(clk1x_u2),  
//               .CLK2X(clk2x_w),  
               .CLKFX(clk4x_w),  
//               .CLKFX180(clk4x180_w),  
               .LOCKED(DCM_L_SYS)
              );


//    IBUFG    VMEINBUF (.I(VME_OSC),.O(vme_in));
//    BUFG    VME_FB (.I(vmeclk1x_w),.O(vmeclkfb_w));
//    BUFGCE VMEBUF (.I(vmeclk1x_w),.CE(VMECE),.O(VME_SYSCLK));
    BUFGCE VMEBUF (.I(VME_OSC),.CE(VMECE),.O(VME_SYSCLK));

`ifdef Simulation
    // Attributes for functional simulation//
    // synthesis translate_off
//       defparam U3_DCM.CLK_FEEDBACK          = "1X";
//       defparam U3_DCM.DLL_FREQUENCY_MODE    = "LOW";
//       defparam U3_DCM.DFS_FREQUENCY_MODE    = "LOW";
//       defparam U3_DCM.DUTY_CYCLE_CORRECTION = "TRUE";
//       defparam U3_DCM.STARTUP_WAIT          = "FALSE";
    // synthesis translate_on
`endif


    // Instantiate the DCM primitive//
//         DCM U3_DCM ( 
//               .CLKIN(vme_in), 
//               .CLKFB(vmeclkfb_w), 
//               .DSSEN(GND), 
//               .PSCLK(GND), 
//               .PSEN(GND), 
//               .PSINCDEC(GND), 
//               .RST(DCM_RST), 
//               .CLK0(vmeclk1x_w),  
//               .LOCKED(DCM_L_VME)
//              );
// attribute declarations for synthesis
//synthesis attribute          CLK_FEEDBACK of U3_DCM is "1X" 
//synthesis attribute    DLL_FREQUENCY_MODE of U3_DCM is "LOW" 
//synthesis attribute    DFS_FREQUENCY_MODE of U3_DCM is "LOW" 
//synthesis attribute DUTY_CYCLE_CORRECTION of U3_DCM is "TRUE" 
//synthesis attribute          STARTUP_WAIT of U3_DCM is "FALSE" 


    BUFG    SERFBBUF (.I(serclk1x_w),.O(serfb_w));

    BUFG    SERBUF (.I(serclk_w),.O(SER_CLK));

`ifdef Simulation
    // Attributes for functional simulation//
    // synthesis translate_off
       defparam U4_DCM.CLK_FEEDBACK          = "1X";
       defparam U4_DCM.DLL_FREQUENCY_MODE    = "LOW";
       defparam U4_DCM.DFS_FREQUENCY_MODE    = "LOW";
       defparam U4_DCM.CLKDV_DIVIDE          = 8;
       defparam U4_DCM.DUTY_CYCLE_CORRECTION = "TRUE";
       defparam U4_DCM.STARTUP_WAIT          = "FALSE";
    // synthesis translate_on
`endif

// attribute declarations for synthesis
//synthesis attribute          CLK_FEEDBACK of U4_DCM is "1X" 
//synthesis attribute    DLL_FREQUENCY_MODE of U4_DCM is "LOW" 
//synthesis attribute    DFS_FREQUENCY_MODE of U4_DCM is "LOW" 
//synthesis attribute          CLKDV_DIVIDE of U4_DCM is "8" 
//synthesis attribute DUTY_CYCLE_CORRECTION of U4_DCM is "TRUE" 
//synthesis attribute          STARTUP_WAIT of U4_DCM is "FALSE"

    // Instantiate the DCM primitive//
         DCM U4_DCM ( 
               .CLKIN(REFCLK), 
               .CLKFB(serfb_w), 
               .DSSEN(GND), 
               .PSCLK(GND), 
               .PSEN(GND), 
               .PSINCDEC(GND), 
               .RST(DCM_RST), 
               .CLK0(serclk1x_w),  
               .CLKDV(serclk_w),  
               .LOCKED(DCM_L_SER)
              );


//always @(posedge SER_CLK or posedge DCM_RST)
//begin
//   if(DCM_RST)
//      DV_CNT_DWN <= 0;
//   else 
//      DV_CNT_DWN <= DV_CNT_DWN-1;
//end
//
//BUFG SLOW_CLK(.I(DV_CNT_DWN[6]),.O(DV1024CLK));

SRL16E DV32_CLK        (.CLK(USRCLK),.CE(!POW_ON_RST),.D(!DV32C),.Q(DV32C),.A3(1'b1),.A2(1'b1),.A1(1'b1),.A0(1'b1)); // 16 bits divide by 32
BUFG   DV32_CLK_BUF    (.I(DV32C),.O(DV32CLK));
SRL16E DV1024_CLK      (.CLK(DV32CLK),.CE(!POW_ON_RST),.D(!DV1024C),.Q(DV1024C),.A3(1'b1),.A2(1'b1),.A1(1'b1),.A0(1'b1));// 16 bits divide by 32
BUFG   DV1024_CLK_BUF  (.I(DV1024C),.O(DV1024CLK));

SRL16E IM_CLK      (.CLK(DV1024CLK),.CE(!POW_ON_RST),.D(!IMC),.Q(IMC),.A3(1'b0),.A2(1'b1),.A1(1'b1),.A0(1'b0));   // 6 bits divide by 12
BUFG   IM_CLK_BUF  (.I(IMC),.O(IMCLK));
SRL16E MS_CLK      (.CLK(IMCLK),.CE(!POW_ON_RST),.D(!MSC),.Q(MSC),.A3(1'b1),.A2(1'b0),.A1(1'b1),.A0(1'b0));   // 10 bits divide by 20
BUFG   MS_CLK_BUF  (.I(MSC),.O(CK_3P9MS));

endmodule
