///////////////////////////////////////////////////////////////
//							     //
// FIFOS for Normal and Priority MAC Adresses                //
//							     //
///////////////////////////////////////////////////////////////

`timescale 1ns / 1ps

module norm_prio_mac(CLK,RXRESET,MAC_DEST,PRIO_FRAME,PUSH,NORM_MAC,NORM_POP,
                        NORM_MT,PRIO_MAC,PRIO_POP,PRIO_MT);
    input CLK;
    input RXRESET;
    input [15:0] MAC_DEST;
    input PRIO_FRAME;
    input PUSH;
    output [15:0] NORM_MAC;
    input NORM_POP;
    output NORM_MT;
    output [15:0] PRIO_MAC;
    input PRIO_POP;
    output PRIO_MT;

reg NORM_CE;
reg NORM_DIR;
reg N_POP;
reg PRIO_CE;
reg PRIO_DIR;
reg P_POP;

wire NORM_PUSH;
wire PRIO_PUSH;
wire [9:0] NORM_WADDR;
wire [9:0] NORM_RADDR;
wire [9:0] NORM_CNT;
wire [9:0] PRIO_WADDR;
wire [9:0] PRIO_RADDR;
wire [9:0] PRIO_CNT;

`ifdef HEADER
`else
`include "header.v"
`endif


/*
// Comparison constants
`define   Zero_2  10'd0     // 10 bit zero
*/
assign NORM_PUSH = (PUSH && !PRIO_FRAME);
assign   NORM_MT = (NORM_CNT == `Zero_2);
assign PRIO_PUSH = (PUSH && PRIO_FRAME);
assign   PRIO_MT = (PRIO_CNT == `Zero_2);

///////////////////////////////////////////////////////////////
//							     //
// FIFO for Normal MAC Adresses			             //
//							     //
///////////////////////////////////////////////////////////////
 
defparam  norm_MAC_wrt_ptr.width = 10;
defparam norm_MAC_read_ptr.width = 10;
defparam norm_cntr.width = 10;

cnt_up norm_MAC_wrt_ptr (.clk(CLK),.inc(NORM_PUSH),.r(RXRESET),.count(NORM_WADDR));
// 1k X 16 Dual Port RAM for MAC addresses
dpm_1kX16 norm_MAC_fifo(.clk(CLK),.we(NORM_PUSH),.a(NORM_WADDR),.dpra(NORM_RADDR),.di(MAC_DEST),.dpo(NORM_MAC)); 
cnt_up  norm_MAC_read_ptr (.clk(CLK),.inc(N_POP),.r(RXRESET),.count(NORM_RADDR));

cnt_up_dwn norm_cntr (.clk(CLK),.ce(NORM_CE),.dir(NORM_DIR),.r(RXRESET),.count(NORM_CNT));

always @ (NORM_PUSH or NORM_POP or NORM_MT)
begin
  casex ({NORM_MT,NORM_PUSH,NORM_POP})
     3'b001:
       begin		  // count down
         NORM_CE <= 1'b1;
	 NORM_DIR <= 1'b0;
         N_POP <= 1'b1;	 // Increment address
       end
     3'bx10,3'b111:
       begin
         NORM_CE <= 1'b1;	  // count up
	 NORM_DIR <= 1'b1;
         N_POP <= 1'b0;	 // Don't increment address
       end
     3'b011:
       begin		  // hold
         NORM_CE <= 1'b0;
	 NORM_DIR <= 1'b1;
         N_POP <= 1'b1;	 // Increment address
       end
     default:	          // hold
       begin
         NORM_CE <= 1'b0;
	 NORM_DIR <= 1'b1;
         N_POP <= 1'b0;
       end
  endcase
end

///////////////////////////////////////////////////////////////
//							     //
// FIFO for Priority MAC Adresses		             //
//							     //
///////////////////////////////////////////////////////////////
 
defparam  prio_MAC_wrt_ptr.width = 10;
defparam prio_MAC_read_ptr.width = 10;
defparam prio_cntr.width = 10;

cnt_up prio_MAC_wrt_ptr (.clk(CLK),.inc(PRIO_PUSH),.r(RXRESET),.count(PRIO_WADDR));
// 1k X 16 Dual Port RAM for MAC addresses
dpm_1kX16 prio_MAC_fifo(.clk(CLK),.we(PRIO_PUSH),.a(PRIO_WADDR),.dpra(PRIO_RADDR),.di(MAC_DEST),.dpo(PRIO_MAC)); 
cnt_up  prio_MAC_read_ptr (.clk(CLK),.inc(P_POP),.r(RXRESET),.count(PRIO_RADDR));

cnt_up_dwn prio_cntr (.clk(CLK),.ce(PRIO_CE),.dir(PRIO_DIR),.r(RXRESET),.count(PRIO_CNT));

always @ (PRIO_PUSH or PRIO_POP or PRIO_MT)
begin
  casex ({PRIO_MT,PRIO_PUSH,PRIO_POP})
     3'b001:
       begin		  // count down
         PRIO_CE <= 1'b1;
	 PRIO_DIR <= 1'b0;
         P_POP <= 1'b1;	  // Increment address
       end
     3'bx10,3'b111:
       begin
         PRIO_CE <= 1'b1;	  // count up
	 PRIO_DIR <= 1'b1;
         P_POP <= 1'b0;	  // Don't increment address
       end
     3'b011:
       begin		  // hold
         PRIO_CE <= 1'b0;
	 PRIO_DIR <= 1'b1;
         P_POP <= 1'b1;	  // Increment address
       end
     default:	          // hold
       begin
         PRIO_CE <= 1'b0;
	 PRIO_DIR <= 1'b1;
         P_POP <= 1'b0;
       end
  endcase
end


endmodule
